<!-- Improved compatibility of back to top link: See: https://github.com/othneildrew/Best-README-Template/pull/73 -->
<a id="readme-top"></a>
<!--

<!-- PROJECT LOGO -->

<div align="center">
    <img src="images/BSS Logo Black on White.png" alt="Logo" width="240" height="109">
  

<h3 align="center">ECG Monitor Firmware</h3>

  <p align="center">
    Firmware for Arduino-like hardware to use TCO ECG Monitor without Labview.

  


  </p>
</div>




<!-- TABLE OF CONTENTS -->
<details>
  <summary>Table of Contents</summary>
  <ol>
    <li><a href="#about-the-project">About The Project</a></li>
    <li><a href="#getting-started">Getting Started</a></li>
    <li><a href="#Hardware modifications">Hardware modifications</a></li>	
	<li><a href="#Installation">Installation</a></li>
	<li><a href="#Concept of the code">Concept of the code</a></li>
    <li><a href="#roadmap">Roadmap</a></li>
    <li><a href="#license">License</a></li>
    <li><a href="#contact">Contact</a></li>
    <li><a href="#acknowledgments">Acknowledgments</a></li>
  </ol>
</details>



<!-- ABOUT THE PROJECT -->
## About The Project

<div align="center">
 <img src="images/screenshot_uscope.png" alt="screen shot uScope" >
</div> 

In 2008 TCO developped an ECG monitor based upon hardware design and requirements from the BSS group. 
Hardware is sufficient, but software compatibility is increasingly troublesome. 
The USB interface between the ECG monitor and the PC is based upon a special NI Labview driver. 
This driver is not compatible with recent security features which are enabled by default on current laptops. 
Disabling these security features is not always feasible, and in any case not wanted. 

Since the hardware is still functional, and “good enough” with respect to the measurement quality, 
we decided to only modify the interface between the ECG monitor and the PC and leave the analog part 
and Analog/Digital converter (AD7683) in place. 

In the practical assignment of the Biorobotics Minor, a modular hardware platform based upon µPython is used. 
Part of this platform is a simple program to view the EMG signals: “<a href="https://bitbucket.org/ctw-bw/uscope/src/master/">uScope</a>”. 
For the ECG monitor, the implemented data format allows use this same uScope signal viewer. 



<p align="right">(<a href="#readme-top">back to top</a>)</p>



<!-- GETTING STARTED -->
## Getting Started

Charge the battery of the TCO Monitor. 
Power on the TCO monitor. Connect ECG electrodes. 
Connect the USB cable to the PC.
Download uScope for Windows or OS X on your PC, via the download section of the <a href="https://bitbucket.org/ctw-bw/uscope/src/master/">repository</a>.

Start uScope. In the drop down menu select a "COM - Serial Device". Press connect, and the graph should start scrolling with a yellow line.

If not, try other devices in the pulldown menu. 
You can verify the COM port number in the Windows device manager under the "Ports (COM & LPT)" group. Simply unplug and plug the ECG monitor to see which device appears.


<!-- Hardware modifications -->
### Hardware modifications

The TCO ECG monitor needs to be modified once to allow usage of this new firmware. This should only be performed by a experienced engineer.

-	Remove PIC18F2550 processor
-	Solder wires to the resistors that interface to the ADC chip
	- IC2 and IC4 provide large pins on which you can solder the wires. 
		- IC2 Pin 6 is D_OUT of the battery monitor. Should be connected to Teensy 34 DAT1 (MISO of the µSD interface of the Teensy). FOR FUTURE USE, not implemented yet.
		- IC2 Pin 7 is D_OUT of the ADC. Should be connected to Teensy GPIO 12 (MISO)
		- IC4 Pin 2 is D_CLK of ADC and Batt monitor. Should be connected to Teensy GPIO 13 (SCK)
		- IC4 Pin 2 is Chip Select of ADC and Batt monitor. Should be connected to Teensy GPIO 10 (CS)
	- Header CON3 provides the 5V for the Teensy. Pin 4 (closest to the edge) is GND and Pin 3 is 5V DC. Connect them to the "G" and "5V" pins on the corners of the Teensy.
-	Physically mount the Teensy 4.0 in the ECG Monitor
-	Wire the USB interface
	- Remove the 5 position Dupont housing from the connector pins of the microUSB cable.
	- Insert the cables into 2x3 housing according to figure below:
	 <img src="images/USB_Pinout.png" alt="USB Pinout">

-	Unrelated, but good to do when the device is open:
	-	Replace battery

<!-- Installation -->
### Installation

Teensy should have the correct firmware in place. If it does not identify as USB Serial Device, you can reprogram it.

For development Visual Studio Code was used, in combination with PlatformIO. This should automatically load the required dependencies as specified in the PlatformIO.ini file.

The "Compiled Firmware" folder contains the HEX and ELF files. These can be flashed to the Teensy 4.0 using the Teensy Loader. (see PJRC.com) 


<p align="right">(<a href="#readme-top">back to top</a>)</p>


<!-- Concept of the code -->
### Concept of the code

Code is Arduino based. 

A task scheduler by https://github.com/arkhipenko/TaskScheduler/ is used to time aquisition and data sending individually.

The aquisitionTask samples data from the ADC, and creates a binary sample set (including the microsecond counter). 
This sample set is appended to the either send buffer A or B, depending on which buffer is locked by the communicationTask. 

The communicationTask locks one buffer before it sends it to the serial receiver. Via this mechanism, the communicationTask and aquisitionTask never use the same buffer at the same time, avoiding conflicts.
The const long communicationTaskDuration = 10000; variable defines that 100 times per second the data is send to the serial receiver.

Because the raw signals from the AD7683 turned out to be noisy, an oversampling mechanism is implemented. 
const uint8_t downSampler = 16; defines the number of samples from the ADC wich are averaged before sending them to the serial receiver. 
const long aquisitionTaskDuration = 125; defines that the ADC is sampled at 8 kHz. Since 16 samples are averaged, the resulting output frequency is 500 Hz.

The downsampler just calculates the mean of the last 16 samples, no filtering is applied. 

The code contains a debug log function with different verbose levels. It should always be on silent:
const int verboseLevel = helpers::verboseLevels::silent; supresses all log output which would otherwise interfere with the sample data.


<p align="right">(<a href="#readme-top">back to top</a>)</p>




<!-- ROADMAP -->
## Roadmap


- Battery Montior
	- The TCO Monitor contains two ADCs, one for the ECG and one for the battery. The Clock and Chip Select lines are shared, and both have their own MISO lines. This violates the normal SPI conventions. 
	The PIC processor in the original design accepted this, but the Teensy doesn't want to clock a second SPI interface using the clock of the first. Sometimes garbarge is received. 
	The Batt Low led on the device is purely analog, and remains functional.

- USB HID
	- Current firmware uses USB virtual COM port to communicate. HID class USB should also be feasible without installing custom drivers, and allows for much higher data rates. There is a fork of uScope for HID.

- µPython support
    - One of the unique features of the Biorobotics platform is the ability to run simple Python scripts on the embedded hardware. 
	By choosing hardware (Teensy 4.0) which is compatible with µPython, we leave this possibility open. 
	However, the Analogue-Digital convertor is not yet supported in µPython, software modifications will be required before the ECG monitor can be used with µPython.


See the [open issues](https://gitlab.utwente.nl/bss_development/neuro/tco-ecg-monitor/-/issues) for a full list known issues and feature requests.

<p align="right">(<a href="#readme-top">back to top</a>)</p>






<!-- LICENSE -->
## License

Distributed under Creative Commons License CC BY-SA

This license enables reusers to distribute, remix, adapt, and build upon the material in any medium or format, so long as attribution is given to the creator. 
The license allows for commercial use. If you remix, adapt, or build upon the material, you must license the modified material under identical terms. 
CC BY-SA includes the following elements:
 BY: credit must be given to the creator.
 SA: Adaptations must be shared under the same terms.



<p align="right">(<a href="#readme-top">back to top</a>)</p>



<!-- CONTACT -->
## Contact

Frodo Muijzer 

<p align="right">(<a href="#readme-top">back to top</a>)</p>



<!-- ACKNOWLEDGMENTS -->
## Acknowledgments

This readme is based upon: [github.com/othneildrew/Best-README-Template](https://github.com/othneildrew/Best-README-Template)
The hardware is created by TCO - University of Twente. 

<p align="right">(<a href="#readme-top">back to top</a>)</p>


