
#include <InstrumentedObjectHelpers.h>
const int verboseLevel = helpers::verboseLevels::silent; //define what output should be send to the USB serial console.

namespace helpers
{
  //returns the current static verbose level, all messages with an higher or equal level will be printed to the log
  extern const int getVerboseLevel()
  {
    return verboseLevel;
  }
  
  //converts to a string "> INFO: " etc.
  extern String verboseLevelToString(int verboseLevelToConvert)
  {
    switch (verboseLevelToConvert)
    {
    case verboseLevels::error:
      return "> ERROR: ";
    case verboseLevels::info:
      return "> INFO: ";
    case verboseLevels::extraBT:
      return "> EXTRA BT: ";
    case verboseLevels::extraSensor:
      return "> EXTRA SENSOR: ";
    case verboseLevels::extraTiming:
      return "> EXTRA TIMING: ";
    case verboseLevels::extraBuffers:
      return "> EXTRA TIMING: ";

    default:
      return "> (unknown level): ";
    }
  }

  //if the set verbose level is equal or higher than the verbose level of the individual message, it will be printed to the USB serial console
  extern void printToLog(String messsage, int verboseLevelOfThisMessage)
  {
    String verboseLevelStr = verboseLevelToString(verboseLevelOfThisMessage);
    if (getVerboseLevel() >= verboseLevelOfThisMessage)
    {
      Serial.print(millis());        //log current time
      Serial.print(verboseLevelStr); // "> ERROR: " or "> INFO: "
      Serial.println(messsage);      //write out the message
    }   
  }
  //if the set verbose level is equal or higher than the verbose level of the individual message, it will be printed to the USB serial console
  //overloaded version that enables the error led upon error. If LED is not available, set errorLedGPIO to -1 to suppress the error led
  extern void printToLog(String messsage, int verboseLevelOfThisMessage, int errorLedGPIO)
  {
    printToLog(messsage, verboseLevelOfThisMessage);
    if (verboseLevelOfThisMessage == verboseLevels::error) //error message, turn on error led if avaiable
    {
      if (errorLedGPIO >= 0) //led is indicated, if LED is not available, set it to -1 to suppress the error led
      {                                  
        digitalWrite(errorLedGPIO, LOW); //show error
      }
    }
  }

 

}